"""
The :mod:`pyswarms.base` module implements base
swarm classes to implement variants of particle swarm optimization.
"""

from .base_single import SwarmBase
from .base_discrete import DiscreteSwarmBase

__all__ = [
    "SwarmBase",
    "DiscreteSwarmBase"
    ]



# -*- coding: utf-8 -*-

r"""
A Local-best Particle Swarm Optimization (lbest PSO) algorithm.

Similar to global-best PSO, it takes a set of candidate solutions,
and finds the best solution using a position-velocity update method.
However, it uses a ring topology, thus making the particles
attracted to its corresponding neighborhood.

The position update can be defined as:

.. math::

   x_{i}(t+1) = x_{i}(t) + v_{i}(t+1)

Where the position at the current timestep :math:`t` is updated using
the computed velocity at :math:`t+1`. Furthermore, the velocity update
is defined as:

.. math::

   v_{ij}(t + 1) = m * v_{ij}(t) + c_{1}r_{1j}(t)[y_{ij}(t) − x_{ij}(t)] + c_{2}r_{2j}(t)[\hat{y}_{j}(t) − x_{ij}(t)]

However, in local-best PSO, a particle doesn't compare itself to the
overall performance of the swarm. Instead, it looks at the performance
of its nearest-neighbours, and compares itself with them. In general,
this kind of topology takes much more time to converge, but has a more
powerful explorative feature.

In this implementation, a neighbor is selected via a k-D tree
imported from :code:`scipy`. Distance are computed with either
the L1 or L2 distance. The nearest-neighbours are then queried from
this k-D tree.

An example usage is as follows:

.. code-block:: python

    import pyswarms as ps
    from pyswarms.utils.functions import single_obj as fx

    # Set-up hyperparameters
    options = {'c1': 0.5, 'c2': 0.3, 'w': 0.9, 'k': 3, 'p': 2}

    # Call instance of LBestPSO with a neighbour-size of 3 determined by
    # the L2 (p=2) distance.
    optimizer = ps.single.LocalBestPSO(n_particles=10, dimensions=2,
                                       options=options)

    # Perform optimization
    stats = optimizer.optimize(fx.sphere_func, iters=100)

This algorithm was adapted from one of the earlier works of
J. Kennedy and R.C. Eberhart in Particle Swarm Optimization
[IJCNN1995]_ [MHS1995]

.. [IJCNN1995] J. Kennedy and R.C. Eberhart, "Particle Swarm Optimization,"
    Proceedings of the IEEE International Joint Conference on Neural
    Networks, 1995, pp. 1942-1948.

.. [MHS1995] J. Kennedy and R.C. Eberhart, "A New Optimizer using Particle
    Swarm Theory,"  in Proceedings of the Sixth International
    Symposium on Micromachine and Human Science, 1995, pp. 39–43.
"""

# Import from __future__
from __future__ import with_statement
from __future__ import absolute_import
from __future__ import print_function

# Import modules
import logging
import numpy as np
from scipy.spatial import cKDTree
from past.builtins import xrange

# Import from package
from ..base import SwarmBase
from ..utils.console_utils import cli_print, end_report


class LocalBestPSO(SwarmBase):

    def assertions(self):
        """Assertion method to check various inputs.

        Raises
        ------
        KeyError
            When one of the required dictionary keys is missing.
        ValueError
            When the number of neighbors is not within the range
                :code:`[0, n_particles]`.
            When the p-value is not in the list of values :code:`[1,2]`.
        """
        super(LocalBestPSO, self).assertions()

        if not all(key in self.options for key in ('k', 'p')):
            raise KeyError('Missing either k or p in options')
        if not 0 <= self.k <= self.n_particles:
            raise ValueError('No. of neighbors must be between 0 and no. '
                             'of particles.')
        if self.p not in [1, 2]:
            raise ValueError('p-value should either be 1 (for L1/Minkowski) '
                             'or 2 (for L2/Euclidean).')

    def __init__(self, n_particles, dimensions, options, bounds=None,
                 velocity_clamp=None, init_pos=None, ftol=-np.inf):
        """Initializes the swarm.

        Attributes
        ----------
        n_particles : int
            number of particles in the swarm.
        dimensions : int
            number of dimensions in the space.
        bounds : tuple of np.ndarray, optional (default is :code:`None`)
            a tuple of size 2 where the first entry is the minimum bound
            while the second entry is the maximum bound. Each array must
            be of shape :code:`(dimensions,)`.
        velocity_clamp : tuple (default is :code:`None`)
            a tuple of size 2 where the first entry is the minimum velocity
            and the second entry is the maximum velocity. It
            sets the limits for velocity clamping.
        init_pos : list (default is :code:`None`)
            an array of size :code:`dimensions`
        ftol : float
            relative error in objective_func(best_pos) acceptable for
            convergence
        options : dict with keys :code:`{'c1', 'c2', 'w', 'k', 'p'}`
            a dictionary containing the parameters for the specific
            optimization technique
                * c1 : float
                    cognitive parameter
                * c2 : float
                    social parameter
                * w : float
                    inertia parameter
                * k : int
                    number of neighbors to be considered. Must be a
                    positive integer less than :code:`n_particles`
                * p: int {1,2}
                    the Minkowski p-norm to use. 1 is the
                    sum-of-absolute values (or L1 distance) while 2 is
                    the Euclidean (or L2) distance.
        """
        # Initialize logger
        self.logger = logging.getLogger(__name__)
        # Assign k-neighbors and p-value as attributes
        self.k, self.p = options['k'], options['p']
        # Initialize parent class
        super(LocalBestPSO, self).__init__(n_particles, dimensions, options,
                                           bounds, velocity_clamp, init_pos, ftol)
        # Invoke assertions
        self.assertions()
        # Initialize the resettable attributes
        self.reset()

    def optimize(self, objective_func, iters, print_step=1, verbose=1):
        """Optimizes the swarm for a number of iterations.

        Performs the optimization to evaluate the objective
        function :code:`f` for a number of iterations :code:`iter.`

        Parameters
        ----------
        objective_func : function
            objective function to be evaluated
        iters : int
            number of iterations
        print_step : int (default is 1)
            amount of steps for printing into console.
        verbose : int  (default is 1)
            verbosity setting.

        Returns
        -------
        tuple
            the local best cost and the local best position among the
            swarm.
        """
        for i in xrange(iters):
            # Compute cost for current position and personal best
            current_cost = objective_func(self.pos)
            pbest_cost = objective_func(self.personal_best_pos)

            # Store current best cost found
            best_cost_yet_found = np.min(self.best_cost)
            
            # Update personal bests if the current position is better
            # Create a 1-D mask then update pbest_cost
            m = (current_cost < pbest_cost)
            pbest_cost = np.where(~m, pbest_cost, current_cost)
            # Create a 2-D mask to update positions
            _m = np.repeat(m[:, np.newaxis], self.dimensions, axis=1)
            self.personal_best_pos = np.where(~_m, self.personal_best_pos,
                                              self.pos)

            # Obtain the indices of the best position for each
            # neighbour-space, and get the local best cost and
            # local best positions from it.
            nmin_idx = self._get_neighbors(pbest_cost)
            self.best_cost = pbest_cost[nmin_idx]
            self.best_pos = self.personal_best_pos[nmin_idx]

            # Print to console
            if i % print_step == 0:
                cli_print('Iteration %s/%s, cost: %s' %
                          (i+1, iters, np.min(self.best_cost)), verbose, 2,
                          logger=self.logger)

            # Save to history
            hist = self.ToHistory(
                best_cost=np.min(self.best_cost),
                mean_pbest_cost=np.mean(pbest_cost),
                mean_neighbor_cost=np.mean(self.best_cost),
                position=self.pos,
                velocity=self.velocity
            )
            self._populate_history(hist)

            # Verify stop criteria based on the relative acceptable cost ftol
            relative_measure = self.ftol*(1 + np.abs(best_cost_yet_found))
            if np.abs(np.min(self.best_cost) - best_cost_yet_found) < relative_measure:
                break
            # Perform position velocity update
            self._update_velocity()
            self._update_position()

        # Obtain the final best_cost and the final best_position
        final_best_cost_arg = np.argmin(self.best_cost)
        final_best_cost = np.min(self.best_cost)
        final_best_pos = self.best_pos[final_best_cost_arg]

        end_report(final_best_cost, final_best_pos, verbose,
                   logger=self.logger)
        return final_best_cost, final_best_pos

    def _get_neighbors(self, pbest_cost):
        """Helper function to obtain the best position found in the
        neighborhood. This uses the cKDTree method from :code:`scipy`
        to obtain the nearest neighbours

        Parameters
        ----------
        pbest_cost : numpy.ndarray of size :code:`(n_particles, )`
            the cost incurred at the historically best position. Will be used
            for mapping the obtained indices to its actual cost.

        Returns
        -------
        array of size (n_particles, ) dtype=int64
            indices containing the best particles for each particle's
            neighbour-space that have the lowest cost
        """
        # Use cKDTree to get the indices of the nearest neighbors
        tree = cKDTree(self.pos)
        _, idx = tree.query(self.pos, p=self.p, k=self.k)

        # Map the computed costs to the neighbour indices and take the
        # argmin. If k-neighbors is equal to 1, then the swarm acts
        # independently of each other.
        if self.k == 1:
            # The minimum index is itself, no mapping needed.
            best_neighbor = pbest_cost[idx][:, np.newaxis].argmin(axis=1)
        else:
            idx_min = pbest_cost[idx].argmin(axis=1)
            best_neighbor = idx[np.arange(len(idx)), idx_min]

        return best_neighbor

    def _update_velocity(self):
        """Updates the velocity matrix of the swarm.

        This method updates the attribute :code:`self.velocity` of
        the instantiated object. It is called by the
        :code:`self.optimize()` method.
        """
        # Define the hyperparameters from options dictionary
        c1, c2, w = self.options['c1'], self.options['c2'], self.options['w']

        # Compute for cognitive and social terms
        cognitive = (c1 * np.random.uniform(0, 1, self.swarm_size)
                     * (self.personal_best_pos - self.pos))
        social = (c2 * np.random.uniform(0, 1, self.swarm_size)
                  * (self.best_pos - self.pos))
        temp_velocity = (w * self.velocity) + cognitive + social

        # Create a mask to clamp the velocities
        if self.velocity_clamp is not None:
            # Create a mask depending on the set boundaries
            min_velocity, max_velocity = self.velocity_clamp[0], \
                                         self.velocity_clamp[1]
            _b = np.logical_and(temp_velocity >= min_velocity,
                                temp_velocity <= max_velocity)
            # Use the mask to finally clamp the velocities
            self.velocity = np.where(~_b, self.velocity, temp_velocity)
        else:
            self.velocity = temp_velocity

    def _update_position(self):
        """Updates the position matrix of the swarm.

        This method updates the attribute :code:`self.pos` of
        the instantiated object. It is called by the
        :code:`self.optimize()` method.
        """
        # Update position and store it in a temporary variable
        temp = self.pos.copy()
        temp += self.velocity

        if self.bounds is not None:
            # Create a mask depending on the set boundaries
            b = (np.all(self.min_bounds <= temp, axis=1)
                 * np.all(temp <= self.max_bounds, axis=1))
            # Broadcast the mask
            b = np.repeat(b[:, np.newaxis], self.dimensions, axis=1)
            # Use the mask to finally guide position update
            temp = np.where(~b, self.pos, temp)
        self.pos = temp



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Unit testing for pyswarms.single.LocalBestPSO"""

# Import from __future__
from __future__ import with_statement
from __future__ import absolute_import
from __future__ import print_function

# Import modules
import unittest
import numpy as np

# Import from package
from pyswarms.single import LocalBestPSO
from pyswarms.utils.functions.single_obj import sphere_func

class Base(unittest.TestCase):
    """Base class for tests

    This class defines a common `setUp` method that defines attributes
    which are used in the various tests.
    """
    def setUp(self):
        """Set up test fixtures"""
        self.options = {'c1':0.5, 'c2':0.7, 'w':0.5, 'k': 2, 'p': 2}
        self.safe_bounds = (np.array([-5,-5]), np.array([5,5]))
        self.optimizer = LocalBestPSO(10,2, options=self.options)


class Instantiation(Base):
    """Tests all aspects of instantiation

    Tests include: instantiation with args of wrong type, instantiation
    with input values outside constraints, etc.
    """
    def test_keyword_check_fail(self):
        """Tests if exceptions are thrown when keywords are missing"""
        check_c1 = {'c2':0.7, 'w':0.5, 'k': 2, 'p': 2}
        check_c2 = {'c1':0.5, 'w':0.5, 'k': 2, 'p': 2}
        check_m = {'c1':0.5, 'c2':0.7, 'k': 2, 'p': 2}
        check_k = {'c1':0.5, 'c2':0.7, 'w':0.5, 'p': 2}
        check_p = {'c1':0.5, 'c2':0.7, 'w':0.5, 'k': 2}
        with self.assertRaises(KeyError):
            optimizer = LocalBestPSO(5,2,options=check_c1)
        with self.assertRaises(KeyError):
            optimizer = LocalBestPSO(5,2,options=check_c2)
        with self.assertRaises(KeyError):
            optimizer = LocalBestPSO(5,2,options=check_m)
        with self.assertRaises(KeyError):
            optimizer = LocalBestPSO(5,2,options=check_k)
        with self.assertRaises(KeyError):
            optimizer = LocalBestPSO(5,2,options=check_p)

    def test_bound_size_fail(self):
        """Tests if exception is thrown when bound length is not 2"""
        bounds = tuple(np.array([-5,-5]))
        with self.assertRaises(IndexError):
            optimizer = LocalBestPSO(5,2, bounds=bounds, options=self.options)

    def test_bound_type_fail(self):
        """Tests if exception is thrown when bound type is not tuple"""
        bounds = [np.array([-5,-5]), np.array([5,5])]
        with self.assertRaises(TypeError):
            optimizer = LocalBestPSO(5,2, bounds=bounds, options=self.options)

    def test_bound_maxmin_fail(self):
        """Tests if exception is thrown when min max of the bound is
        wrong."""
        bounds_1 = (np.array([5,5]), np.array([-5,-5]))
        bounds_2 = (np.array([5,-5]), np.array([-5,5]))
        with self.assertRaises(ValueError):
            optimizer = LocalBestPSO(5,2, bounds=bounds_1, options=self.options)
        with self.assertRaises(ValueError):
            optimizer = LocalBestPSO(5,2, bounds=bounds_2, options=self.options)

    def test_bound_shapes_fail(self):
        """Tests if exception is thrown when bounds are of unequal
        shapes."""
        bounds = (np.array([-5,-5,-5]), np.array([5,5]))
        with self.assertRaises(IndexError):
            optimizer = LocalBestPSO(5,2, bounds=bounds, options=self.options)

    def test_bound_shape_dims_fail(self):
        """Tests if exception is thrown when bound shape is not equal
        to dimensions."""
        bounds = (np.array([-5,-5,-5]), np.array([5,5,5]))
        with self.assertRaises(IndexError):
            optimizer = LocalBestPSO(5,2, bounds=bounds, options=self.options)

    def test_k_fail(self):
        """Tests if exception is thrown when feeding an invalid k."""
        k_less_than_min = {'c1':0.5, 'c2':0.7, 'w':0.5, 'k':-1, 'p':2}
        k_more_than_max = {'c1':0.5, 'c2':0.7, 'w':0.5, 'k':6, 'p':2}

        with self.assertRaises(ValueError):
            optimizer = LocalBestPSO(5,2, bounds=self.safe_bounds, options=k_less_than_min)
        with self.assertRaises(ValueError):
            optimizer = LocalBestPSO(5,2, bounds=self.safe_bounds, options=k_more_than_max)

    def test_p_fail(self):
        """Tests if exception is thrown when feeding an invalid p."""
        p_fail = {'c1':0.5, 'c2':0.7, 'w':0.5, 'k':2, 'p':5}
        with self.assertRaises(ValueError):
            optimizer = LocalBestPSO(5,2, options=p_fail)

    def test_vclamp_type_fail(self):
        """Tests if exception is thrown when velocity_clamp is not a tuple."""
        velocity_clamp = [1,3]
        with self.assertRaises(TypeError):
            optimizer = LocalBestPSO(5,2,velocity_clamp=velocity_clamp, options=self.options)

    def test_vclamp_shape_fail(self):
        """Tests if exception is thrown when velocity_clamp is not equal to 2"""
        velocity_clamp = (1,1,1)
        with self.assertRaises(IndexError):
            optimizer = LocalBestPSO(5,2,velocity_clamp=velocity_clamp, options=self.options)

    def test_vclamp_minmax_fail(self):
        """Tests if exception is thrown when velocity_clamp's minmax is wrong"""
        velocity_clamp = (3,2)
        with self.assertRaises(ValueError):
            optimizer = LocalBestPSO(5,2,velocity_clamp=velocity_clamp, options=self.options)

    def test_init_pos_type_fail(self):
        """Tests if exception is thrown when init_pos is not a list."""
        init_pos = (0.1,1.5)
        with self.assertRaises(TypeError):
            optimizer = LocalBestPSO(5, 2, init_pos=init_pos, options=self.options)

    def test_init_pos_shape_fail(self):
        """Tests if exception is thrown when init_pos dimension is not equal
        to dimensions"""
        init_pos = [1.5, 3.2, 2.5]
        with self.assertRaises(IndexError):
            optimizer = LocalBestPSO(5, 2, init_pos=init_pos, options=self.options)

class MethodsStateChange(Base):
    """Tests all state changes that resulted from method calls"""

    def test_reset_best_cost_inf(self):
        """Tests if best cost is set to infinity when reset() is called"""
        # Perform a simple optimization
        optimizer = LocalBestPSO(5,2, options=self.options)
        optimizer.optimize(sphere_func, 100, verbose=0)

        optimizer.reset()
        self.assertEqual(optimizer.best_cost, np.inf)

    def test_reset_best_pos_none(self):
        """Tests if best pos is set to NoneType when reset() is called"""
        # Perform a simple optimization
        optimizer = LocalBestPSO(5,2, options=self.options)
        optimizer.optimize(sphere_func, 100, verbose=0)

        optimizer.reset()
        self.assertIsNone(optimizer.best_pos)

class RunOptimize(Base):
    """Perform a single run of the algorithm to see if something breaks."""

    def test_run_optimize(self):
        """Perform a single run."""
        try:
            self.optimizer.optimize(sphere_func, 1000, verbose=0)
            trigger = True
        except:
            print('Execution failed.')
            trigger = False

        self.assertTrue(trigger)

    def test_cost_history_size(self):
        """Check the size of the cost_history."""
        self.optimizer.optimize(sphere_func, 1000, verbose=0)
        cost_hist = self.optimizer.get_cost_history
        self.assertEqual(cost_hist.shape, (1000,))

    def test_mean_pbest_history_size(self):
        """Check the size of the mean_pbest_history."""
        self.optimizer.optimize(sphere_func, 1000, verbose=0)
        mean_pbest_hist = self.optimizer.get_mean_pbest_history
        self.assertEqual(mean_pbest_hist.shape, (1000,))

    def test_mean_neighbor_history_size(self):
        """Check the size of the mean neighborhood history."""
        self.optimizer.optimize(sphere_func, 1000, verbose=0)
        mean_neighbor_hist = self.optimizer.get_mean_neighbor_history
        self.assertEqual(mean_neighbor_hist.shape, (1000,))

    def test_pos_history_size(self):
        """Check the size of the pos_history."""
        self.optimizer.optimize(sphere_func, 1000, verbose=0)
        pos_hist = self.optimizer.get_pos_history
        self.assertEqual(pos_hist.shape, (1000, 10, 2))

    def test_velocity_history_size(self):
        """Check the size of the velocity_history."""
        self.optimizer.optimize(sphere_func, 1000, verbose=0)
        velocity_hist = self.optimizer.get_velocity_history
        self.assertEqual(velocity_hist.shape, (1000, 10, 2))

    def test_ftol_effect(self):
        """Check if setting ftol breaks the optimization process
        accordingly."""
        # Perform a simple optimization
        optimizer = LocalBestPSO(10,2, options=self.options, ftol=1e-1)
        optimizer.optimize(sphere_func, 5000, verbose=0)
        cost_hist = optimizer.get_cost_history
        self.assertNotEqual(cost_hist.shape, (5000, ))

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

"""
Particle Swarm Optimization (PSO) toolkit
=========================================
PySwarms is a particle swarm optimization (PSO) toolkit that enables
researchers to test variants of the PSO technique in different contexts.
Users can define their own function, or use one of the benchmark functions
in the library. It is built on top of :code:`numpy` and :code:`scipy`, and
is very extensible to accommodate other PSO variations.
"""

__author__ = """Lester James V. Miranda"""
__email__ = 'ljvmiranda@gmail.com'
__version__ = '0.1.9'

from .single import global_best, local_best
from .discrete import binary

__all__ = [
    'global_best',
    'local_best',
    'binary'
    ]



# -*- coding: utf-8 -*-

r"""
Base class for single-objective discrete Particle Swarm Optimization
implementations.

All methods here are abstract and raises a :code:`NotImplementedError`
when not used. When defining your own swarm implementation,
create another class,

    >>> class MySwarm(DiscreteSwarmBase):
    >>>     def __init__(self):
    >>>        super(MySwarm, self).__init__()

and define all the necessary methods needed.

As a guide, check the discrete PSO implementations in this package.

.. note:: Regarding :code:`options`, it is highly recommended to
    include parameters used in position and velocity updates as
    keyword arguments. For parameters that affect the topology of
    the swarm, it may be much better to have them as positional
    arguments.

See Also
--------
:mod:`pyswarms.discrete.binary`: binary PSO implementation

"""

import os
import yaml
import logging
import numpy as np
import logging.config
from collections import namedtuple


class DiscreteSwarmBase(object):

    def assertions(self):
        """Assertion method to check various inputs.

        Raises
        ------
        TypeError
            When the :code:`bounds` is not of type tuple
        IndexError
            When the :code:`bounds` is not of size 2.
            When the arrays in :code:`bounds` is not of equal size.
            When the shape of :code:`bounds` is not the same as `dimensions`.
        ValueError
            When the value of :code:`bounds[1]` is less than
            :code:`bounds[0]`.
        """

        # Check clamp settings
        if self.velocity_clamp is not None:
            if not isinstance(self.velocity_clamp, tuple):
                raise TypeError('Parameter `velocity_clamp` must be a tuple')
            if not len(self.velocity_clamp) == 2:
                raise IndexError('Parameter `velocity_clamp` must be of '
                                 'size 2')
            if not self.velocity_clamp[0] < self.velocity_clamp[1]:
                raise ValueError('Make sure that velocity_clamp is in the '
                                 'form (v_min, v_max)')

        # Required keys in options argument
        if not all(key in self.options for key in ('c1', 'c2', 'w')):
            raise KeyError('Missing either c1, c2, or w in options')

    def setup_logging(self, default_path='./config/logging.yaml',
                      default_level=logging.INFO, env_key='LOG_CFG'):
        """Setup logging configuration

        Parameters
        ----------
        default_path : str (default is `./config/logging.yaml`)
            the path where the logging configuration is stored
        default_level: logging.LEVEL (default is `logging.INFO`)
            the default logging level
        env_key : str
            the environment key for accessing the setup
        """
        path = default_path
        value = os.getenv(env_key, None)
        if value:
            path = value
        if os.path.exists(path):
            with open(path, 'rt') as f:
                config = yaml.safe_load(f.read())
            logging.config.dictConfig(config)
        else:
            logging.basicConfig(level=default_level)

    def __init__(self, n_particles, dimensions, binary, options,
                 velocity_clamp=None):
        """Initializes the swarm.

        Creates a :code:`numpy.ndarray` of positions depending on the
        number of particles needed and the number of dimensions.
        The initial positions of the particles depends on the argument
        :code:`binary`, which governs if a binary matrix will be produced.

        Attributes
        ----------
        n_particles : int
            number of particles in the swarm.
        dimensions : int
            number of dimensions in the space.
        binary : boolean
            a trigger to generate a binary matrix for the swarm's
            initial positions. When passed with a :code:`False` value,
            random integers from 0 to :code:`dimensions` are generated.
        options : dict with keys :code:`{'c1', 'c2', 'w'}`
            a dictionary containing the parameters for the specific 
            optimization technique
                * c1 : float
                    cognitive parameter
                * c2 : float
                    social parameter
                * w : float
                    inertia parameter
        velocity_clamp : tuple (default is :code:`None`)
            a tuple of size 2 where the first entry is the minimum velocity
            and the second entry is the maximum velocity. It
            sets the limits for velocity clamping.
        options: dict
            a dictionary containing the parameters for a specific
            optimization technique
        """
        self.setup_logging()
        # Initialize primary swarm attributes
        self.n_particles = n_particles
        self.dimensions = dimensions
        self.binary = binary
        self.velocity_clamp = velocity_clamp
        self.swarm_size = (n_particles, dimensions)
        self.options = options
        # Initialize named tuple for populating the history list
        self.ToHistory = namedtuple('ToHistory',
                                    ['best_cost', 'mean_pbest_cost',
                                     'mean_neighbor_cost', 'position',
                                     'velocity'])
        # Invoke assertions
        self.assertions()

        # Initialize resettable attributes
        self.reset()

    def _populate_history(self, hist):
        """Populates all history lists

        The :code:`cost_history`, :code:`mean_pbest_history`, and
        :code:`neighborhood_best` is expected to have a shape of
        :code:`(iters,)`,on the other hand, the :code:`pos_history`
        and :code:`velocity_history` are expected to have a shape of
        :code:`(iters, n_particles, dimensions)`

        Parameters
        ----------
        hist : namedtuple
            Must be of the same type as self.ToHistory
        """
        self.cost_history.append(hist.best_cost)
        self.mean_pbest_history.append(hist.mean_pbest_cost)
        self.mean_neighbor_history.append(hist.mean_neighbor_cost)
        self.pos_history.append(hist.position)
        self.velocity_history.append(hist.velocity)

    @property
    def get_cost_history(self):
        """Get cost history"""
        return np.array(self.cost_history)

    @property
    def get_mean_pbest_history(self):
        """Get mean personal best history"""
        return np.array(self.mean_pbest_history)

    @property
    def get_mean_neighbor_history(self):
        """Get mean neighborhood cost history"""
        return np.array(self.mean_neighbor_history)

    @property
    def get_pos_history(self):
        """Get position history"""
        return np.array(self.pos_history)

    @property
    def get_velocity_history(self):
        """Get velocity history"""
        return np.array(self.velocity_history)

    def optimize(self, objective_func, iters, print_step=1, verbose=1):
        """Optimizes the swarm for a number of iterations.

        Performs the optimization to evaluate the objective
        function :code:`objective_func` for a number of iterations
                 :code:`iter.`

        Parameters
        ----------
        objective_func : function
            objective function to be evaluated
        iters : int
            number of iterations
        print_step : int (the default is 1)
            amount of steps for printing into console.
        verbose : int (the default is 1)
            verbosity setting.

        Raises
        ------
        NotImplementedError
            When this method is not implemented.
        """
        raise NotImplementedError("SwarmBase::optimize()")

    def _update_velocity(self):
        """Updates the velocity matrix.

        Raises
        ------
        NotImplementedError
            When this method is not implemented.
        """
        raise NotImplementedError("SwarmBase::_update_velocity()")

    def _update_position(self):
        """Updates the position matrix.

        Raises
        ------
        NotImplementedError
            When this method is not implemented.
        """
        raise NotImplementedError("SwarmBase::_update_position()")

    def reset(self):
        """Resets the attributes of the optimizer.

        All variables/atributes that will be re-initialized when this
        method is defined here. Note that this method
        can be called twice: (1) during initialization, and (2) when
        this is called from an instance.

        It is good practice to keep the number of resettable
        attributes at a minimum. This is to prevent spamming the same
        object instance with various swarm definitions.

        Normally, swarm definitions are as atomic as possible, where
        each type of swarm is contained in its own instance. Thus, the
        following attributes are the only ones recommended to be
        resettable:
            * Swarm position matrix (self.pos)
            * Velocity matrix (self.pos)
            * Best scores and positions (gbest_cost, gbest_pos, etc.)

        Otherwise, consider using positional arguments.
        """
        # Initialize history lists
        self.cost_history = []
        self.mean_pbest_history = []
        self.mean_neighbor_history = []
        self.pos_history = []
        self.velocity_history = []

        # Generate initial position
        self.pos = np.random.random_sample(size=self.swarm_size).\
            argsort(axis=1)
        if self.binary:
            self.pos = np.random.randint(2, size=self.swarm_size)

        # Initialize velocity vectors
        if self.velocity_clamp is not None:
            min_velocity, max_velocity = self.velocity_clamp[0],\
                                         self.velocity_clamp[1]
            self.velocity = ((max_velocity - min_velocity)
                             * np.random.random_sample(size=self.swarm_size)
                             + min_velocity)
        else:
            self.velocity = np.random.random_sample(size=self.swarm_size)

        # Initialize the best cost of the swarm
        self.best_cost = np.inf
        self.best_pos = None

        # Initialize the personal best of each particle
        self.personal_best_pos = self.pos



# -*- coding: utf-8 -*-

"""single_obj.py: collection of single-objective functions

All objective functions :code:`obj_func()` must accept a
:code:`(numpy.ndarray)` with shape :code:`(n_particles, dimensions)`.
Thus, each row represents a  particle, and each column represents its
position on a specific dimension of the search-space.

In this context, :code:`obj_func()` must return an array :code:`j`
of size :code:`(n_particles, )` that contains all the computed fitness
for each particle.

Whenever you make changes to this file via an implementation
of a new objective function, be sure to perform unittesting
in order to check if all functions implemented adheres to
the design pattern stated above.
"""

# Import from __future__
from __future__ import with_statement
from __future__ import absolute_import
from __future__ import print_function

# Import modules
import numpy as np


def sphere_func(x):
    """Sphere objective function.

    Has a global minimum at :code:`0` and with a search domain of
        :code:`[-inf, inf]`

    Parameters
    ----------
    x : numpy.ndarray
        set of inputs of shape :code:`(n_particles, dimensions)`

    Returns
    -------
    numpy.ndarray
        computed cost of size :code:`(n_particles, )`
    """
    j = (x**2.0).sum(axis=1)

    return j


def rastrigin_func(x):
    """Rastrigin objective function.

    Has a global minimum at :code:`f(0,0,...,0)` with a search
    domain of :code:`[-5.12, 5.12]`

    Parameters
    ----------
    x : numpy.ndarray
        set of inputs of shape :code:`(n_particles, dimensions)`

    Returns
    -------
    numpy.ndarray
        computed cost of size :code:`(n_particles, )`

    Raises
    ------
    ValueError
        When the input is out of bounds with respect to the function
        domain
    """
    if not np.logical_and(x >= -5.12, x <= 5.12).all():
        raise ValueError('Input for Rastrigin function must be within '
                         '[-5.12, 5.12].')

    d = x.shape[1]
    j = 10.0 * d + (x**2.0 - 10.0 * np.cos(2.0 * np.pi * x)).sum(axis=1)

    return j


def ackley_func(x):
    """Ackley's objective function.

    Has a global minimum at :code:`f(0,0,...,0)` with a search
    domain of [-32, 32]

    Parameters
    ----------
    x : numpy.ndarray
        set of inputs of shape :code:`(n_particles, dimensions)`

    Returns
    -------
    numpy.ndarray
        computed cost of size :code:`(n_particles, )`

    Raises
    ------
    ValueError
        When the input is out of bounds with respect to the function
        domain
    """
    if not np.logical_and(x >= -32, x <= 32).all():
        raise ValueError('Input for Ackley function must be within [-32, 32].')

    d = x.shape[1]
    j = (-20.0 * np.exp(-0.2 * np.sqrt((1/d) * (x**2).sum(axis=1)))
         - np.exp((1/float(d)) * np.cos(2 * np.pi * x).sum(axis=1))
         + 20.0
         + np.exp(1))

    return j


def rosenbrock_func(x):
    """Rosenbrock objective function.

    Also known as the Rosenbrock's valley or Rosenbrock's banana
    function. Has a global minimum of :code:`np.ones(dimensions)` where
    :code:`dimensions` is :code:`x.shape[1]`. The search domain is
    :code:`[-inf, inf]`.

    Parameters
    ----------
    x : numpy.ndarray
        set of inputs of shape :code:`(n_particles, dimensions)`

    Returns
    -------
    numpy.ndarray
        computed cost of size :code:`(n_particles, )`
    """

    r = np.sum(100*(x.T[1:] - x.T[:-1]**2.0)**2 + (1-x.T[:-1])**2.0, axis=0)

    return r


def beale_func(x):
    """Beale objective function.

    Only takes two dimensions and has a global minimum at
    :code:`f([3,0.5])` Its domain is bounded between :code:`[-4.5, 4.5]`

    Parameters
    ----------
    x : numpy.ndarray
        set of inputs of shape :code:`(n_particles, dimensions)`

    Returns
    -------
    numpy.ndarray
        computed cost of size :code:`(n_particles, )`

    Raises
    ------
    IndexError
        When the input dimensions is greater than what the function
        allows
    ValueError
        When the input is out of bounds with respect to the function
        domain
    """
    if not x.shape[1] == 2:
        raise IndexError('Beale function only takes two-dimensional input.')
    if not np.logical_and(x >= -4.5, x <= 4.5).all():
        raise ValueError('Input for Beale function must be within '
                         '[-4.5, 4.5].')

    x_ = x[:, 0]
    y_ = x[:, 1]
    j = ((1.5 - x_ + x_ * y_)**2.0
         + (2.25 - x_ + x_ * y_**2.0)**2.0
         + (2.625 - x_ + x_ * y_**3.0)**2.0)

    return j


# TODO: Implement Goldstein-Price's Function
def goldstein_func(x):
    """Goldstein-Price's objective function.

    Only takes two dimensions and has a global minimum at
    :code:`f([0,-1])`. Its domain is bounded between :code:`[-2, 2]`

    Parameters
    ----------
    x : numpy.ndarray
        set of inputs of shape :code:`(n_particles, dimensions)`

    Returns
    -------
    numpy.ndarray
        computed cost of size :code:`(n_particles, )`

    Raises
    ------
    IndexError
        When the input dimensions is greater than what the function
        allows
    ValueError
        When the input is out of bounds with respect to the function
        domain
    """
    if not x.shape[1] == 2:
        raise IndexError('Goldstein function only takes two-dimensional '
                         'input.')
    if not np.logical_and(x >= -2, x <= 2).all():
        raise ValueError('Input for Goldstein-Price function must be within '
                         '[-2, 2].')

    x_ = x[:, 0]
    y_ = x[:, 1]
    j = ((1 + (x_ + y_ + 1)**2.0
         * (19 - 14*x_ + 3*x_**2.0 - 14*y_ + 6*x_*y_ + 3*y_**2.0))
         * (30 + (2*x_ - 3 * y_)**2.0
         * (18 - 32*x_ + 12*x_**2.0 + 48*y_ - 36*x_*y_ + 27*y_**2.0)))

    return j


# TODO: Implement Booth's Function
def booth_func(x):
    """Booth's objective function.

    Only takes two dimensions and has a global minimum at
    :code:`f([1,3])`. Its domain is bounded between :code:`[-10, 10]`

    Parameters
    ----------
    x : numpy.ndarray
        set of inputs of shape :code:`(n_particles, dimensions)`

    Returns
    -------
    numpy.ndarray
        computed cost of size :code:`(n_particles, )`

    Raises
    ------
    IndexError
        When the input dimensions is greater than what the function
        allows
    ValueError
        When the input is out of bounds with respect to the function
        domain
    """
    if not x.shape[1] == 2:
        raise IndexError('Booth function only takes two-dimensional input.')
    if not np.logical_and(x >= -10, x <= 10).all():
        raise ValueError('Input for Booth function must be within [-10, 10].')

    x_ = x[:, 0]
    y_ = x[:, 1]
    j = (x_ + 2 * y_ - 7)**2.0 + (2 * x_ + y_ - 5)**2.0

    return j


# TODO: Implement Bukin Function no. 6
def bukin6_func(x):
    """Bukin N. 6 Objective Function

    Only takes two dimensions and has a global minimum at
    :code:`f([-10,1])`. Its coordinates are bounded by:
        * x[:,0] must be within [-15, -5]
        * x[:,1] must be within [-3, 3]

    Parameters
    ----------
    x : numpy.ndarray
        set of inputs of shape :code:`(n_particles, dimensions)`

    Returns
    -------
    numpy.ndarray
        computed cost of size :code:`(n_particles, )`

    Raises
    ------
    IndexError
        When the input dimensions is greater than what the function
        allows
    ValueError
        When the input is out of bounds with respect to the function
        domain
    """
    if not x.shape[1] == 2:
        raise IndexError('Bukin N. 6 function only takes two-dimensional '
                         'input.')
    if not np.logical_and(x[:, 0] >= -15, x[:, 0] <= -5).all():
        raise ValueError('x-coord for Bukin N. 6 function must be within '
                         '[-15, -5].')
    if not np.logical_and(x[:, 1] >= -3, x[:, 1] <= 3).all():
        raise ValueError('y-coord for Bukin N. 6 function must be within '
                         '[-3, 3].')

    x_ = x[:, 0]
    y_ = x[:, 1]
    j = 100 * np.sqrt(np.absolute(y_**2.0 - 0.01*x_**2.0)) + 0.01 * \
        np.absolute(x_ + 10)

    return j


def matyas_func(x):
    """Matyas objective function

    Only takes two dimensions and has a global minimum at
    :code:`f([0,0])`. Its coordinates are bounded within
    :code:`[-10,10]`.

    Parameters
    ----------
    x : numpy.ndarray
        set of inputs of shape :code:`(n_particles, dimensions)`

    Returns
    -------
    numpy.ndarray
    """
    if not x.shape[1] == 2:
        raise IndexError('Matyas function only takes two-dimensional input.')
    if not np.logical_and(x >= -10, x <= 10).all():
        raise ValueError('Input for Matyas function must be within '
                         '[-10, 10].')

    x_ = x[:, 0]
    y_ = x[:, 1]
    j = 0.26 * (x_**2.0 + y_**2.0) - 0.48 * x_ * y_

    return j


def levi_func(x):
    """Levi objective function

    Only takes two dimensions and has a global minimum at
    :code:`f([1,1])`. Its coordinates are bounded within
    :code:`[-10,10]`.

    Parameters
    ----------
    x : numpy.ndarray
        set of inputs of shape :code:`(n_particles, dimensions)`

    Returns
    -------
    numpy.ndarray
        computed cost of size :code:`(n_particles, )`

    Raises
    ------
    IndexError
        When the input dimensions is greater than what the function
        allows
    ValueError
        When the input is out of bounds with respect to the function
        domain
    """
    if not x.shape[1] == 2:
        raise IndexError('Levi function only takes two-dimensional input.')
    if not np.logical_and(x >= -10, x <= 10).all():
        raise ValueError('Input for Levi function must be within [-10, 10].')

    mask = np.full(x.shape, False)
    mask[:, -1] = True
    masked_x = np.ma.array(x, mask=mask)

    w_ = 1 + (x - 1) / 4
    masked_w_ = np.ma.array(w_, mask=mask)
    d_ = x.shape[1] - 1

    j = (np.sin(np.pi * w_[:, 0])**2.0
         + ((masked_x - 1)**2.0).sum(axis=1)
         * (1 + 10 * np.sin(np.pi * (masked_w_).sum(axis=1) + 1)**2.0)
         + (w_[:, d_] - 1)**2.0
         * (1 + np.sin(2 * np.pi * w_[:, d_])**2.0))

    return j


def schaffer2_func(x):
    """Schaffer N.2 objective function

    Only takes two dimensions and has a global minimum at
    :code:`f([0,0])`. Its coordinates are bounded within
    :code:`[-100,100]`.

    Parameters
    ----------
    x : numpy.ndarray
        set of inputs of shape :code:`(n_particles, dimensions)`

    Returns
    -------
    numpy.ndarray
        computed cost of size :code:`(n_particles, )`

    Raises
    ------
    IndexError
        When the input dimensions is greater than what the function
        allows
    ValueError
        When the input is out of bounds with respect to the function
        domain
    """
    if not x.shape[1] == 2:
        raise IndexError('Schaffer N. 2 function only takes two-dimensional '
                         'input.')
    if not np.logical_and(x >= -100, x <= 100).all():
        raise ValueError('Input for Schaffer function must be within '
                         '[-100, 100].')

    x_ = x[:, 0]
    y_ = x[:, 1]
    j = (0.5
         + ((np.sin(x_**2.0 - y_**2.0)**2.0 - 0.5)
             / ((1 + 0.001 * (x_**2.0 + y_**2.0))**2.0)))

    return j



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Import from __future__
from __future__ import with_statement
from __future__ import absolute_import
from __future__ import print_function

# Import modules
import os
import unittest
import numpy as np
from mock import Mock
import matplotlib as mpl
from matplotlib.axes._subplots import SubplotBase
from matplotlib.animation import FuncAnimation

# Fix for display
if os.environ.get('DISPLAY','') == '':
    mpl.use('Agg')

# Import from package
from pyswarms.utils.environments import PlotEnvironment
from pyswarms.single import GlobalBestPSO
from pyswarms.utils.functions.single_obj import sphere_func

class Base(unittest.TestCase):

    def setUp(self):
        """Sets up test fixtures"""
        self.optimizer = GlobalBestPSO(n_particles=10,dimensions=3,
            options={'c1':0.5,'c2':0.3,'w':0.9})
        self.class_methods = [
            'get_cost_history',
            'get_pos_history',
            'get_velocity_history',
            'optimize',
            'reset']
        self.get_specs = lambda idx: [x for i,x in enumerate(self.class_methods) if i!=idx]
        self.plt_env = PlotEnvironment(self.optimizer, sphere_func, 1000)

class Instantiation(Base):

    def test_objective_function_fail(self):
        """Tests if exception is thrown when objective function is not callable"""
        pass_a_list = [1, 2, 3, 4, 5]
        pass_a_dict = {'a':0, 'b':2, 'c':4}
        with self.assertRaises(TypeError):
            plt_env = PlotEnvironment(self.optimizer, pass_a_list, 100)
        with self.assertRaises(TypeError):
            plt_env = PlotEnvironment(self.optimizer, pass_a_dict, 100)

    def test_optimizer_getters_get_cost_history_fail(self):
        """Test if exception is thrown if get_cost_history is missing"""
        m = Mock(spec=self.get_specs(0))
        with self.assertRaises(AttributeError):
            plt_env = PlotEnvironment(m, sphere_func, 100)

    def test_optimizer_getters_get_pos_history_fail(self):
        """Test if exception is thrown if get_cost_history is missing"""
        m = Mock(spec=self.get_specs(1))
        with self.assertRaises(AttributeError):
            plt_env = PlotEnvironment(m, sphere_func, 100)

    def test_optimizer_getters_get_velo_history_fail(self):
        """Test if exception is thrown if get_velocity_history is missing"""
        m = Mock(spec=self.get_specs(2))
        with self.assertRaises(AttributeError):
            plt_env = PlotEnvironment(m, sphere_func, 100)

    def test_attribute_optimize_fail(self):
        """Test if exception is thrown if optimize attr is missing"""
        m = Mock(spec=self.get_specs(3))
        with self.assertRaises(AttributeError):
            plt_env = PlotEnvironment(m, sphere_func, 100)

    def test_attribute_reset_fail(self):
        """Test if exception is thrown if reset attr is missing"""
        m = Mock(spec=self.get_specs(4))
        with self.assertRaises(AttributeError):
            plt_env = PlotEnvironment(m, sphere_func, 100)

class MethodReturnTypes(Base):

    def test_plot_cost_return_type(self):
        """Test if plot_cost() returns a SubplotBase instance"""
        # It turns out that AxesSubplot is just a factory-created class,
        # that's why checking on it explicitly is impossible.
        self.assertIsInstance(self.plt_env.plot_cost(), SubplotBase)

    def test_plot2D_return_type(self):
        """Test if plot_particles2D() returns a FuncAnimation instance"""
        self.assertIsInstance(self.plt_env.plot_particles2D(), FuncAnimation)

    def test_plot3D_return_type(self):
        """Test if plot_particles3D() returns a FuncAnimation instance"""
        self.assertIsInstance(self.plt_env.plot_particles3D(), FuncAnimation)


# -*- coding: utf-8 -*-
"""
Hyperparameter grid search.

Compares the relative performance of hyperparameter value combinations in
optimizing a specified objective function.

For each hyperparameter, user can provide either a single value or a list
of possible values. The cartesian products of these hyperparameters are taken
to produce a grid of all possible combinations. These combinations are then
tested to produce a list of objective function scores. The search method
default returns the minimum objective function score and hyperparameters that
yield the minimum score, yet maximum score can also be evaluated.

>>> options = {'c1': [1, 2, 3],
               'c2': [1, 2, 3],
               'w' : [2, 3, 5],
               'k' : [5, 10, 15],
               'p' : 1}
>>> g = GridSearch(LocalBestPSO, n_particles=40, dimensions=20,
                   options=options, objective_func=sphere_func, iters=10)
>>> best_score, best_options = g.search()
>>> best_score
0.498641604188
>>> best_options['c1']
1
>>> best_options['c2']
1
"""

# Import from __future__
from __future__ import with_statement
from __future__ import absolute_import
from __future__ import print_function

# Import modules
import itertools

# Import from package
from pyswarms.utils.search.base_search import SearchBase


class GridSearch(SearchBase):
    """Exhaustive search of optimal performance on selected objective function
    over all combinations of specified hyperparameter values."""

    def __init__(self, optimizer, n_particles, dimensions, options,
                 objective_func, iters, bounds=None, velocity_clamp=None):
        """Initializes the paramsearch."""

        # Assign attributes
        super(GridSearch, self).__init__(optimizer, n_particles, dimensions,
                                         options, objective_func, iters,
                                         bounds=bounds,
                                         velocity_clamp=velocity_clamp)
        # invoke assertions
        self.assertions()

    def generate_grid(self):
        """Generates the grid of all hyperparameter value combinations."""

        # Extract keys and values from options dictionary
        params = self.options.keys()
        items = [x if type(x) == list
                 else [x] for x in list(zip(*self.options.items()))[1]]

        # Create list of cartesian products of hyperparameter values
        # from options
        list_of_products = list(itertools.product(*items))

        # Return list of dicts for all hyperparameter value combinations
        return [dict(zip(*[params, list(x)])) for x in list_of_products]



# -*- coding: utf-8 -*-

r"""
Plot environment for Optimizer Analysis

The class PlotEnvironment is built on top of :code:`matplotlib` in order
to render quick and easy plots for your optimizer. It can plot the best
cost for each iteration, and show animations of the particles in 2-D and
3-D space. Furthermore, because it has :code:`matplotlib` running under
the hood, the plots are easily customizable.

For example, if we want to plot the cost using PlotEnvironment, simply
pass the optimizer object when initializing the class, and the
PlotEnvironment will do a fresh run of your optimizer. After that,
various plotting methods can now be done:

.. code-block:: python
    import pyswarms as ps
    from pyswarms.utils.functions.single_obj import sphere_func
    from pyswarms.utils.environments import PlotEnvironment

    # Set up optimizer
    options = {'c1':0.5, 'c2':0.3, 'w':0.9}
    optimizer = ps.single.GlobalBestPSO(n_particles=10, dimensions=2,
                                        options=options)

    # Pass optimizer  inside the environment. You also need to pass some
    # of the required arguments on how your optimizer will be evaluated.
    plt_env = PlotEnvironment(optimizer, sphere_func, 1000)

    # To plot the cost
    plt_env.plot_cost()
    plt.show()

In case you want to plot the particle movement, it is important that either
one of the :code:`matplotlib` animation :code:`Writers` is installed. These
doesn't come out of the box for :code:`pyswarms`, and must be installed
separately. For example, in a Linux or Windows distribution, you can
install :code:`ffmpeg` as

    >>> conda install -c conda-forge ffmpeg

Now, if you want to plot your particles in a 2-D environment, simply call
the following function:

    >>> plt_env.plot_particles2d()

You can also supply various arguments in this method: the indices of the
specific dimensions to be used, the limits of the axes, and the interval/
speed of animation.
"""

# Import from __future__
from __future__ import with_statement
from __future__ import absolute_import
from __future__ import print_function

# Import modules
import logging
import numpy as np
import matplotlib.pyplot as plt
from past.builtins import xrange
from matplotlib import animation
from collections import namedtuple
from mpl_toolkits.mplot3d import Axes3D


class PlotEnvironment(object):

    def assertions(self):
        """Assertion check"""
        # Check if the objective_func is a callable
        if not callable(self.objective_func):
            raise TypeError('Must pass a callable')

        # Check if getters exist in the optimizer
        if not (hasattr(self.optimizer, 'get_cost_history')
                & hasattr(self.optimizer, 'get_pos_history')
                & hasattr(self.optimizer, 'get_velocity_history')):
            raise AttributeError('Missing getters in optimizer, check '
                                 'pyswarms.base module')

        # Check if important methods exist in the optimizer
        if not (hasattr(self.optimizer, 'optimize')
                & hasattr(self.optimizer, 'reset')):
            raise AttributeError('Missing methods in optimizer, check '
                                 'pyswarms.base module')

    def __init__(self, optimizer, objective_func, iters):
        """Runs the optimizer against an objective function for a number
        of iterations

        Upon initialization, the :code:`optimize` method of the optimizer
        will be called, passing the arguments :code:`objective_func` and
        :code:`iters`. The results of the optimization scheme is then
        stored as attributes of this class.

        Parameters
        ----------
        optimizer : object instance
            An instance of an optimizer class that was derived from any
            of the :mod:`pyswarms.base` classes.
        objective_func : method
            An objective function to be optimized using the :code:`optimizer`.
            This argument is passed to the :code:`optimize` method of the
            :code:`optimizer`.
        iters : int
            The number of iterations to run the optimizer. This argument
            is passed to the :code:`optimize` method of the :code:`optimizer`.
        """
        self.logger = logging.getLogger(__name__)
        # Store the arguments
        self.optimizer = optimizer
        self.objective_func = objective_func
        self.iters = iters
        # Check assertions
        self.assertions()
        # Run the optimizer
        self.optimizer.reset()
        self.status = self.optimizer.optimize(objective_func, iters, 1, 0)
        # Initialize tuples for particle plotting
        self.Index = namedtuple('Index', ['x', 'y', 'z'])
        self.Limit = namedtuple('Limit', ['x', 'y', 'z'])
        self.Label = namedtuple('Label', ['x', 'y', 'z'])

    def plot_cost(self, title='Cost History', ax=None, figsize=None,
                  title_fontsize="large", text_fontsize="medium", **kwargs):
        """Creates a simple line plot with the cost in the y-axis and
        the iteration at the x-axis

        Parameters
        ----------
        title : str (default is :code:`'Cost History'`)
            The title of the plotted graph.
        ax : :class:`matplotlib.axes.Axes` (default is :code:`None`)
            The axes where the plot is to be drawn. If :code:`None` is
            passed, then the plot will be drawn to a new set of axes.
        figsize : tuple (default is None)
            Sets the size of the plot figure.
        title_fontsize : str or int (default is :code:`large`)
            This is a :class:`matplotlib.axes.Axes` argument that
            specifies the size of the title. Available values are
            ['small', 'medium', 'large'] or integer values.
        text_fontsize : str or int (default is :code:`large`)
            This is a :class:`matplotlib.axes.Axes` argument that
            specifies the size of various texts around the plot.
            Available values are ['small', 'medium', 'large'] or integer
            values.
        **kwargs : dict
            Keyword arguments that are passed as a keyword argument to
            :class:`matplotlib.axes.Axes`

        Returns
        -------
        :class:`matplotlib.axes._subplots.AxesSubplot`
            The axes on which the plot was drawn.
        """
        # Get cost history from the optimizer method
        cost_history = self.optimizer.get_cost_history
        mean_pbest_history = self.optimizer.get_mean_pbest_history
        mean_neighbor_history = self.optimizer.get_mean_neighbor_history

        # If ax is default, then create new plot
        if ax is None:
            fig, ax = plt.subplots(1, 1, figsize=figsize)

        # Plot with self.iters as x-axis and cost_history as
        # y-axis.
        ax.plot(np.arange(self.iters), cost_history, 'k', lw=2,
                label='Best cost')
        ax.plot(np.arange(self.iters), mean_pbest_history, 'k--', lw=2,
                label='Avg. personal best cost')
        ax.plot(np.arange(self.iters), mean_neighbor_history, 'k:', lw=2,
                label='Avg. neighborhood cost')

        # Customize plot depending on parameters
        ax.set_title(title, fontsize=title_fontsize)
        ax.legend(fontsize=text_fontsize)
        ax.set_xlabel('Iterations', fontsize=text_fontsize)
        ax.set_ylabel('Cost', fontsize=text_fontsize)
        ax.tick_params(labelsize=text_fontsize)

        return ax

    def plot_particles2D(self, index=(0, 1), limits=((-1, 1), (-1, 1)),
                         labels=('x-axis', 'y-axis'), interval=80,
                         title='Particle Movement in 2D space',
                         ax=None, figsize=None, title_fontsize="large",
                         text_fontsize="medium"):
        """Creates an animation of particle movement in 2D-space

        Parameters
        ----------
        index : n-tuple (default is :code:`(0,1)`)
            The index in which a specific dimension will be plotted. For
            example, :code:`(idx_1, idx_2)` for two dimensions.
        limits : n-tuple of 2-tuples (default is :code:`((-1,1),(-1,1))`)
            The limits of the x-y axes for 2D. For example,
            :code:`((xmin, xmax),(ymin, ymax))`
        labels : 2-tuple (default is :code:`('x-axis', 'y-axis')`
            Sets the x and y labels of the 2D plot. For example,
            :code:`('label_x_axis', 'label_y_axis')`
        interval : int (default is 80)
            The speed of update, in milliseconds
        title : str (default is :code:`'Particle Movement in 2D space'`)
            The title of the plotted graph.
        ax : :class:`matplotlib.axes.Axes` (default is :code:`None`)
            The axes where the plot is to be drawn. If :code:`None` is
            passed, then the plot will be drawn to a new set of axes.
        figsize : tuple (default is None)
            Sets the size of the plot figure.
        title_fontsize : str or int (default is :code:`large`)
            This is a :class:`matplotlib.axes.Axes` argument that
            specifies the size of the title. Available values are
            ['small', 'medium', 'large'] or integer values.
        text_fontsize : str or int (default is :code:`large`)
            This is a :class:`matplotlib.axes.Axes` argument that
            specifies the size of various texts around the plot.
            Available values are ['small', 'medium', 'large'] or integer
            values.
        **kwargs : dict
            Keyword arguments that are passed as a keyword argument to
            :class:`matplotlib.axes.Axes`

        Returns
        -------
        :class:`matplotlib.animation.FuncAnimation`
            The drawn animation that can be saved to mp4 or other
            third-party tools
        """
        # Check inconsistencies with input
        if not (len(index) == len(limits) == 2):
            raise ValueError('The index and limits should be of length 2')

        # Set-up tuples for plotting environment
        idx = self.Index(x=index[0], y=index[1], z=None)
        lmt = self.Limit(x=limits[0], y=limits[1], z=None)
        lbl = self.Label(x=labels[0], y=labels[1], z=None)

        # If ax is default, then create new plot. Set-up the figure, the
        # acis, and the plot element that we want to animate
        if ax is None:
            fig, ax = plt.subplots(1, 1, figsize=figsize)

        # Set plot title
        ax.set_title(title, fontsize=title_fontsize)

        # Set plot labels
        ax.set_xlabel(lbl.x, fontsize=text_fontsize)
        ax.set_ylabel(lbl.y, fontsize=text_fontsize)

        # Set plot limits
        ax.set_xlim(lmt.x)
        ax.set_ylim(lmt.y)

        # Plot data
        plot = ax.scatter(x=[], y=[], c='red')
        data = self.optimizer.get_pos_history

        # Get the number of iterations
        n_iters = self.optimizer.get_pos_history.shape[0]

        # Perform animation
        anim = animation.FuncAnimation(fig, func=self._animate2D,
                                       frames=xrange(n_iters),
                                       fargs=(data, plot, idx),
                                       interval=interval, blit=True)
        return anim

    def plot_particles3D(self, index=(0, 1, 2),
                         limits=((-1, 1), (-1, 1), (-1, 1)),
                         labels=('x-axis', 'y-axis', 'z-axis'),
                         interval=80,
                         title='Particle Movement in 3D space', ax=None,
                         figsize=None, title_fontsize="large",
                         text_fontsize="medium"):
        """Creates an animation of particle movement in 2D-space

        Parameters
        ----------
        index : n-tuple (default is :code:`(0,1,2)`)
            The index in which a specific dimension will be plotted. For
            example, :code:`(idx_1, idx_2, idx_3)` for three dimensions.
        limits : n-tuple of 2-tuples (default is
                 :code:`((-1,1),(-1,1),(-1,1))`)
            The limits of the x-y axes for 3D. For example,
            :code:`((xmin, xmax),(ymin, ymax))`
        labels : 2-tuple (default is :code:`('x-axis', 'y-axis', 'z-axis')`
            Sets the x and y labels of the 2D plot. For example,
            :code:`('label_x_axis', 'label_y_axis', 'label_z_axis')`
        interval : int (default is 80)
            The speed of update, in milliseconds
        title : str (default is :code:`'Particle Movement in 3D space'`)
            The title of the plotted graph.
        ax : :class:`matplotlib.axes.Axes` (default is :code:`None`)
            The axes where the plot is to be drawn. If :code:`None` is
            passed, then the plot will be drawn to a new set of axes.
        figsize : tuple (default is None)
            Sets the size of the plot figure.
        title_fontsize : str or int (default is :code:`large`)
            This is a :class:`matplotlib.axes.Axes` argument that
            specifies the size of the title. Available values are
            ['small', 'medium', 'large'] or integer values.
        text_fontsize : str or int (default is :code:`large`)
            This is a :class:`matplotlib.axes.Axes` argument that
            specifies the size of various texts around the plot.
            Available values are ['small', 'medium', 'large'] or integer
            values.
        **kwargs : dict
            Keyword arguments that are passed as a keyword argument to
            :class:`matplotlib.axes.Axes`

        Returns
        -------
        :class:`matplotlib.animation.FuncAnimation`
            The drawn animation that can be saved to mp4 or other
            third-party tools
        """
        # Check inconsistencies with input
        if not (len(index) == len(limits) == 3):
            raise ValueError('The index and limits should be of length 3')

        # Set-up tuples for plotting environment
        idx = self.Index(x=index[0], y=index[1], z=index[2])
        lmt = self.Limit(x=limits[0], y=limits[1], z=limits[2])
        lbl = self.Label(x=labels[0], y=labels[1], z=labels[2])

        # If ax is default, then create new plot. Set-up the figure, the
        # acis, and the plot element that we want to animate
        if ax is None:
            fig, ax = plt.subplots(1, 1, figsize=figsize)
            ax = Axes3D(fig)

        # Set plot title
        ax.set_title(title, fontsize=title_fontsize)

        # Set plot axes labels
        ax.set_xlabel(lbl.x, fontsize=text_fontsize)
        ax.set_ylabel(lbl.y, fontsize=text_fontsize)
        ax.set_zlabel(lbl.z, fontsize=text_fontsize)

        # Set plot limits
        ax.set_xlim(lmt.x)
        ax.set_ylim(lmt.y)
        ax.set_zlim(lmt.z)

        # Plot data
        plot = ax.scatter(xs=[], ys=[], zs=[], c='red')
        data = self.optimizer.get_pos_history

        # Get the number of iterations
        n_iters = self.optimizer.get_pos_history.shape[0]

        # Perform animation
        anim = animation.FuncAnimation(fig, func=self._animate3D,
                                       frames=xrange(n_iters),
                                       fargs=(data, plot, idx),
                                       interval=interval)
        return anim

    def _animate2D(self, i, data, plot, idx):
        """Helper animation function that is called seqentially
        :class:`matplotlib.animation.FuncAnimation`

        Parameters
        ----------
        i : int
            Required argument for :code:`matplotlib.animation.FuncAnimation`,
            basis for indexing the current position of the swarm.
        data : numpy.ndarray
            The position matrix where the particles' position
            will be taken from.
        plot : matplotlib.Axes
            The plot environment where the update operations will be drawn
        idx : namedtuple
            The chosen indices for plotting the dimensions

        Returns
        -------
        :class:`matplotlib.artist.Artist`
            iterable of artists
        """
        current_pos = data[i]
        xy = current_pos[:, (idx.x, idx.y)]
        plot.set_offsets(xy)
        return plot,

    def _animate3D(self, i, data, plot, idx):
        """Helper animation function that is called seqentially
        :class:`matplotlib.animation.FuncAnimation`

        Parameters
        ----------
        i : int
            Required argument for :code:`matplotlib.animation.FuncAnimation`,
            basis for indexing the current position of the swarm.
        data : numpy.ndarray
            The position matrix where the particles' position
            will be taken from.
        plot : matplotlib.Axes
            The plot environment where the update operations will be drawn
        idx : namedtuple
            The chosen indices for plotting the dimensions

        Returns
        -------
        :class:`matplotlib.artist.Artist`
            iterable of artists
        """
        current_pos = data[i]
        x, y, z = current_pos[:, idx.x], current_pos[:, idx.y], \
            current_pos[:, idx.z]
        plot._offsets3d = (x, y, z)
        return plot,



"""
The :mod:`pyswarms.utils.search` module implements various techniques in
hyperparameter value optimization.
"""

from .grid_search import GridSearch
from .random_search import RandomSearch

__all__ = [
    "GridSearch",
    "RandomSearch"
    ]



# -*- coding: utf-8 -*-

""" console_utils.py: various tools for printing into console """

# Import from __future__
from __future__ import with_statement
from __future__ import absolute_import
from __future__ import print_function

# Import modules


def cli_print(message, verbosity, threshold, logger):
    """Helper function to print console output

    Parameters
    ----------
    message : str
        the message to be printed into the console
    verbosity : int
        verbosity setting of the user
    threshold : int
        threshold for printing
    logger : logging.getLogger
        logger instance

    """
    if verbosity >= threshold:
        logger.info(message)
    else:
        pass


def end_report(cost, pos, verbosity, logger):
    """Helper function to print a simple report at the end of the
    run. This always has a threshold of 1.

    Parameters
    ----------
    cost : float
        final cost from the optimization procedure.
    pos : numpy.ndarray or list
        best position found
    verbosity : int
        verbosity setting of the user.
    logger : logging.getLogger
        logger instance
    """

    # Cuts the length of the best position if it's too long
    if len(list(pos)) > 3:
        out = ('[ ' + 3 * '{:3f} ' + '...]').format(*list(pos))
    else:
        out = list(pos)

    template = ('================================\n'
                'Optimization finished!\n'
                'Final cost: {:06.4f}\n'
                'Best value: {}\n').format(cost, out)
    if verbosity >= 1:
        logger.info(template)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `pyswarms` package."""

# Import from __future__
from __future__ import with_statement
from __future__ import absolute_import
from __future__ import print_function

# Import modules
import unittest
import numpy as np

# Import from package
from pyswarms.utils.functions import single_obj as fx

class Base(unittest.TestCase):
    """Base class for testing single-objective functions."""

    def setUp(self):
        """Set up test fixtures."""
        # Test swarm with size=3, dimensions=2
        self.input = np.zeros([3,2])
        self.input2 = np.ones([3,2])
        # Erroneous input - dimension
        self.bad_input = np.zeros([3,3])
        # Target value
        self.target = np.zeros(3)
        # Target output size
        self.target_size = (self.input.shape[0], )

class InstantiationBound(Base):
    """Tests exception throws when fed with erroneous input."""

    def test_rastrigin_bound_fail(self):
        """Test rastrigin bound exception"""
        x = - np.random.uniform(low=6,high=100,size=(3,2))
        x_ = np.random.uniform(low=6,high=100,size=(3,2))

        with self.assertRaises(ValueError):
            fx.rastrigin_func(x)

        with self.assertRaises(ValueError):
            fx.rastrigin_func(x_)

    def test_ackley_bound_fail(self):
        """Test ackley bound exception"""
        x = - np.random.uniform(low=32,high=100,size=(3,2))
        x_ = np.random.uniform(low=32,high=100,size=(3,2))

        with self.assertRaises(ValueError):
            fx.ackley_func(x)
        with self.assertRaises(ValueError):
            fx.ackley_func(x_)

    def test_beale_bound_fail(self):
        """Test beale bound exception"""
        x = - np.random.uniform(low=4.6666,high=100,size=(3,2))
        x_ = np.random.uniform(low=4.6666,high=100,size=(3,2))

        with self.assertRaises(ValueError):
            fx.beale_func(x)
        with self.assertRaises(ValueError):
            fx.beale_func(x_)

    def test_goldstein_bound_fail(self):
        """Test goldstein bound exception"""
        x = - np.random.uniform(low=2.00001,high=100,size=(3,2))
        x_ = np.random.uniform(low=2.00001,high=100,size=(3,2))

        with self.assertRaises(ValueError):
            fx.goldstein_func(x)
        with self.assertRaises(ValueError):
            fx.goldstein_func(x_)

    def test_booth_bound_fail(self):
        """Test booth bound exception"""
        x = - np.random.uniform(low=11.00001,high=100,size=(3,2))
        x_ = np.random.uniform(low=11.00001,high=100,size=(3,2))

        with self.assertRaises(ValueError):
            fx.booth_func(x)
        with self.assertRaises(ValueError):
            fx.booth_func(x_)

    def test_bukin6_bound_fail(self):
        """Test bukin6 bound exception"""
        x = - np.random.uniform(low=15.001,high=100,size=(3,2))
        x_ =  np.random.uniform(low=-5.001,high=-3.001,size=(3,2))
        x_1 =  np.random.uniform(low=-3.001,high=-100,size=(3,2))

        with self.assertRaises(ValueError):
            fx.bukin6_func(x)
        with self.assertRaises(ValueError):
            fx.bukin6_func(x_)
        with self.assertRaises(ValueError):
            fx.bukin6_func(x_1)

    def test_matyas_bound_fail(self):
        """Test matyas bound exception"""
        x = - np.random.uniform(low=10.001,high=100,size=(3,2))
        x_ = np.random.uniform(low=10.001,high=100,size=(3,2))

        with self.assertRaises(ValueError):
            fx.matyas_func(x)
        with self.assertRaises(ValueError):
            fx.matyas_func(x_)

    def test_levi_bound_fail(self):
        """Test levi bound exception"""
        x = - np.random.uniform(low=10.001,high=100,size=(3,2))
        x_ = np.random.uniform(low=10.001,high=100,size=(3,2))

        with self.assertRaises(ValueError):
            fx.levi_func(x)
        with self.assertRaises(ValueError):
            fx.levi_func(x_)

    def test_schaffer2_bound_fail(self):
        """Test schaffer2 bound exception"""
        x = - np.random.uniform(low=100.001,high=1000,size=(3,2))
        x_ = np.random.uniform(low=100.001,high=1000,size=(3,2))

        with self.assertRaises(ValueError):
            fx.schaffer2_func(x)
        with self.assertRaises(ValueError):
            fx.schaffer2_func(x_)

class InstantiationDimension(Base):
    """Tests exception throws when fed with erroneous dimension."""

    def test_beale_dim_fail(self):
        """Test beale dim exception"""
        with self.assertRaises(IndexError):
            fx.beale_func(self.bad_input)

    def test_goldstein_dim_fail(self):
        """Test golstein dim exception"""
        with self.assertRaises(IndexError):
            fx.goldstein_func(self.bad_input)

    def test_booth_dim_fail(self):
        """Test booth dim exception"""
        with self.assertRaises(IndexError):
            fx.booth_func(self.bad_input)

    def test_bukin6_dim_fail(self):
        """Test bukin6 dim exception"""
        with self.assertRaises(IndexError):
            fx.bukin6_func(self.bad_input)

    def test_matyas_dim_fail(self):
        """Test matyas dim exception"""
        with self.assertRaises(IndexError):
            fx.matyas_func(self.bad_input)

    def test_levi_dim_fail(self):
        """Test levi dim exception"""
        with self.assertRaises(IndexError):
            fx.levi_func(self.bad_input)

    def test_schaffer2_dim_fail(self):
        """Test schaffer2 dim exception"""
        with self.assertRaises(IndexError):
            fx.schaffer2_func(self.bad_input)

class MethodsReturnValue(Base):
    """Tests if a function outputs a minima if fed with expected argmin."""

    def test_sphere_output(self):
        """Tests sphere function output."""
        self.assertEqual(fx.sphere_func(self.input).all(), self.target.all())

    def test_rastrigin_output(self):
        """Tests rastrigin function output."""
        self.assertEqual(fx.rastrigin_func(self.input).all(), self.target.all())

    def test_ackley_output(self):
        """Tests ackley function output."""
        assert np.isclose(fx.ackley_func(self.input), self.target).all()

    def test_rosenbrock_output(self):
        """Tests rosenbrock function output."""
        self.assertEqual(fx.rosenbrock_func(self.input2).all(),np.zeros(3).all())

    def test_beale_output(self):
        """Tests beale function output."""
        assert np.isclose(fx.beale_func([3, 0.5] * self.input2),
            self.target).all()

    def test_goldstein_output(self):
        """Tests goldstein-price function output."""
        assert np.isclose(fx.goldstein_func([0, -1] * self.input2),
            (3 * np.ones(3))).all()

    def test_booth_output(self):
        """Test booth function output."""
        assert np.isclose(fx.booth_func([1, 3] * self.input2),
            self.target).all()

    def test_bukin6_output(self):
        """Test bukin function output."""
        assert np.isclose(fx.bukin6_func([-10, 1] * self.input2),
            self.target).all()

    def test_bukin6_output(self):
        """Test bukin function output."""
        assert np.isclose(fx.matyas_func(self.input),self.target).all()

    def test_levi_output(self):
        """Test levi function output."""
        assert np.isclose(fx.levi_func(self.input2), self.target).all()

    def test_schaffer2_output(self):
        """Test schaffer2 function output."""
        assert np.isclose(fx.schaffer2_func(self.input), self.target).all()

class MethodsReturnSize(Base):
    """Tests if the output of the function is the same as no. of particles"""

    def test_sphere_output_size(self):
        """Tests sphere output size."""
        self.assertEqual(fx.sphere_func(self.input).shape, self.target_size)

    def test_rastrigin_output_size(self):
        """Tests rastrigin output size."""
        self.assertEqual(fx.rastrigin_func(self.input).shape, self.target_size)

    def test_ackley_output_size(self):
        """Tests ackley output size."""
        self.assertEqual(fx.ackley_func(self.input).shape, self.target_size)

    def test_rosenbrock_output_size(self):
        """Tests rosenbrock output size."""
        self.assertEqual(fx.rosenbrock_func(self.input).shape, self.target_size)

    def test_beale_output_size(self):
        """Tests beale output size."""
        self.assertEqual(fx.beale_func(self.input).shape, self.target_size)

    def test_goldstein_output_size(self):
        """Test goldstein output size."""
        self.assertEqual(fx.goldstein_func(self.input).shape, self.target_size)

    def test_booth_output_size(self):
        """Test booth output size."""
        self.assertEqual(fx.booth_func(self.input).shape, self.target_size)

    def test_bukin6_output_size(self):
        """Test bukin6 output size."""
        self.assertEqual(fx.bukin6_func([-10,0] * self.input2).shape, self.target_size)

    def test_levi_output_size(self):
        """Test levi output size."""
        self.assertEqual(fx.levi_func(self.input).shape, self.target_size)

    def test_schaffer2_output_size(self):
        """Test schaffer2 output size."""
        self.assertEqual(fx.schaffer2_func(self.input).shape, self.target_size)

if __name__ == '__main__':
    unittest.main()


# -*- coding: utf-8 -*-

r"""
A Global-best Particle Swarm Optimization (gbest PSO) algorithm.

It takes a set of candidate solutions, and tries to find the best
solution using a position-velocity update method. Uses a
star-topology where each particle is attracted to the best
performing particle.

The position update can be defined as:

.. math::

   x_{i}(t+1) = x_{i}(t) + v_{i}(t+1)

Where the position at the current timestep :math:`t` is updated using
the computed velocity at :math:`t+1`. Furthermore, the velocity update
is defined as:

.. math::

   v_{ij}(t + 1) = m * v_{ij}(t) + c_{1}r_{1j}(t)[y_{ij}(t) − x_{ij}(t)] + c_{2}r_{2j}(t)[\hat{y}_{j}(t) − x_{ij}(t)]

Here, :math:`c1` and :math:`c2` are the cognitive and social parameters
respectively. They control the particle's behavior in choosing how to
react given two choices: (1) to follow its *personal best* or (2) follow
the swarm's *global best* position. Overall, this dictates if the swarm
is explorative or exploitative in nature. In addition, a parameter
:math:`w` controls the inertia of the swarm's movement.

An example usage is as follows:

.. code-block:: python

    import pyswarms as ps
    from pyswarms.utils.functions import single_obj as fx

    # Set-up hyperparameters
    options = {'c1': 0.5, 'c2': 0.3, 'w':0.9}

    # Call instance of GlobalBestPSO
    optimizer = ps.single.GlobalBestPSO(n_particles=10, dimensions=2,
                                        options=options)

    # Perform optimization
    stats = optimizer.optimize(fx.sphere_func, iters=100)

This algorithm was adapted from the earlier works of J. Kennedy and
R.C. Eberhart in Particle Swarm Optimization [IJCNN1995]_.

.. [IJCNN1995] J. Kennedy and R.C. Eberhart, "Particle Swarm Optimization,"
    Proceedings of the IEEE International Joint Conference on Neural
    Networks, 1995, pp. 1942-1948.
"""

# Import from __future__
from __future__ import with_statement
from __future__ import absolute_import
from __future__ import print_function

# Import modules
import logging
import numpy as np
from past.builtins import xrange

# Import from package
from ..base import SwarmBase
from ..utils.console_utils import cli_print, end_report


class GlobalBestPSO(SwarmBase):

    def __init__(self, n_particles, dimensions, options,
                 bounds=None, velocity_clamp=None, init_pos=None, ftol=-np.inf):
        """Initializes the swarm.

        Attributes
        ----------
        n_particles : int
            number of particles in the swarm.
        dimensions : int
            number of dimensions in the space.
        options : dict with keys :code:`{'c1', 'c2', 'w'}`
            a dictionary containing the parameters for the specific
            optimization technique
                * c1 : float
                    cognitive parameter
                * c2 : float
                    social parameter
                * w : float
                    inertia parameter
        bounds : tuple of :code:`np.ndarray` (default is :code:`None`)
            a tuple of size 2 where the first entry is the minimum bound
            while the second entry is the maximum bound. Each array must
            be of shape :code:`(dimensions,)`.
        velocity_clamp : tuple (default is :code:`None`)
            a tuple of size 2 where the first entry is the minimum velocity
            and the second entry is the maximum velocity. It
            sets the limits for velocity clamping.
        init_pos : list (default is :code:`None`)
            an array of size :code:`dimensions`
        ftol : float
            relative error in objective_func(best_pos) acceptable for
            convergence
        """
        super(GlobalBestPSO, self).__init__(n_particles, dimensions, options,
                                            bounds, velocity_clamp,
                                            init_pos, ftol)

        # Initialize logger
        self.logger = logging.getLogger(__name__)
        # Invoke assertions
        self.assertions()
        # Initialize the resettable attributes
        self.reset()

    def optimize(self, objective_func, iters, print_step=1, verbose=1):
        """Optimizes the swarm for a number of iterations.

        Performs the optimization to evaluate the objective
        function :code:`f` for a number of iterations :code:`iter.`

        Parameters
        ----------
        objective_func : function
            objective function to be evaluated
        iters : int
            number of iterations
        print_step : int (default is 1)
            amount of steps for printing into console.
        verbose : int  (default is 1)
            verbosity setting.

        Returns
        -------
        tuple
            the global best cost and the global best position.
        """
        for i in xrange(iters):
            # Compute cost for current position and personal best
            current_cost = objective_func(self.pos)
            pbest_cost = objective_func(self.personal_best_pos)

            # Store current best cost found
            best_cost_yet_found = self.best_cost
            
            # Update personal bests if the current position is better
            # Create 1-D mask then update pbest_cost
            m = (current_cost < pbest_cost)
            pbest_cost = np.where(~m, pbest_cost, current_cost)
            # Create 2-D mask to update positions
            _m = np.repeat(m[:, np.newaxis], self.dimensions, axis=1)
            self.personal_best_pos = np.where(~_m, self.personal_best_pos,
                                              self.pos)

            # Get the minima of the pbest and check if it's less than
            # the saved gbest
            if np.min(pbest_cost) < self.best_cost:
                self.best_cost = np.min(pbest_cost)
                self.best_pos = self.personal_best_pos[np.argmin(pbest_cost)]

            # Print to console
            if i % print_step == 0:
                cli_print('Iteration %s/%s, cost: %s' %
                          (i+1, iters, self.best_cost), verbose, 2,
                          logger=self.logger)

            # Save to history
            hist = self.ToHistory(
                best_cost=self.best_cost,
                mean_pbest_cost=np.mean(pbest_cost),
                mean_neighbor_cost=self.best_cost,
                position=self.pos,
                velocity=self.velocity
            )
            self._populate_history(hist)

            # Verify stop criteria based on the relative acceptable cost ftol
            relative_measure = self.ftol*(1 + np.abs(best_cost_yet_found))
            if np.abs(self.best_cost - best_cost_yet_found) < relative_measure:
                break

            # Perform velocity and position updates
            self._update_velocity()
            self._update_position()

        # Obtain the final best_cost and the final best_position
        final_best_cost = self.best_cost.copy()  # Make deep copies
        final_best_pos = self.best_pos.copy()

        end_report(final_best_cost, final_best_pos, verbose,
                   logger=self.logger)
        return final_best_cost, final_best_pos

    def _update_velocity(self):
        """Updates the velocity matrix of the swarm.

        This method updates the attribute :code:`self.velocity` of
        the instantiated object. It is called by the
        :code:`self.optimize()` method.
        """
        # Define the hyperparameters from options dictionary
        c1, c2, w = self.options['c1'], self.options['c2'], self.options['w']

        # Compute for cognitive and social terms
        cognitive = (c1 * np.random.uniform(0, 1, self.swarm_size)
                     * (self.personal_best_pos - self.pos))
        social = (c2 * np.random.uniform(0, 1, self.swarm_size)
                     * (self.best_pos - self.pos))
        temp_velocity = (w * self.velocity) + cognitive + social

        # Create a mask to clamp the velocities
        if self.velocity_clamp is not None:
            # Create a mask depending on the set boundaries
            min_velocity, max_velocity = self.velocity_clamp[0], \
                                         self.velocity_clamp[1]
            _b = np.logical_and(temp_velocity >= min_velocity,
                                temp_velocity <= max_velocity)
            # Use the mask to finally clamp the velocities
            self.velocity = np.where(~_b, self.velocity, temp_velocity)
        else:
            self.velocity = temp_velocity

    def _update_position(self):
        """Updates the position matrix of the swarm.

        This method updates the attribute :code:`self.pos` of
        the instantiated object. It is called by the
        :code:`self.optimize()` method.
        """
        # Update position and store it in a temporary variable
        temp = self.pos.copy()
        temp += self.velocity

        if self.bounds is not None:
            # Create a mask depending on the set boundaries
            b = (np.all(self.min_bounds <= temp, axis=1)
                 * np.all(temp <= self.max_bounds, axis=1))
            # Broadcast the mask
            b = np.repeat(b[:, np.newaxis], self.dimensions, axis=1)
            # Use the mask to finally guide position update
            temp = np.where(~b, self.pos, temp)
        self.pos = temp



#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" Unit testing for pyswarms.random_search"""

# Import from __future__
from __future__ import with_statement
from __future__ import absolute_import
from __future__ import print_function

# Import modules
import unittest
import numpy as np

# Import from package
from pyswarms.utils.search.random_search import RandomSearch
from pyswarms.single import LocalBestPSO
from pyswarms.single import GlobalBestPSO
from pyswarms.utils.functions.single_obj import sphere_func

class Base(unittest.TestCase):

    def setUp(self):
        """Sets up test fixtures"""
        self.optimizer = LocalBestPSO
        self.n_particles = 40
        self.dimensions = 20
        self.options = {'c1': [1, 5],
                        'c2': [6, 10],
                        'k' : [11, 15],
                        'w' : [0.4, 0.9],
                        'p' : 1}
        self.bounds = (np.array([-5,-5]), np.array([5,5]))
        self.iters = 10
        self.n_selection_iters = 100
        self.objective_func = sphere_func
        self.g = RandomSearch(self.optimizer, self.n_particles,
                self.dimensions, self.options, self.objective_func,
                self.iters, self.n_selection_iters, self.bounds,
                velocity_clamp=None)
        self.g_unbound = RandomSearch(self.optimizer, self.n_particles,
                self.dimensions, self.options, self.objective_func,
                self.iters, self.n_selection_iters,bounds=None,
                velocity_clamp=None)

class MethodReturnType(Base):

    def test_search_min_best_options_return_type(self):
        """Tests if best options returns a dictionary"""
        minimum_best_score, minimum_best_options = self.g_unbound.search()
        self.assertIsInstance(minimum_best_options, dict)

    def test_search_max_best_options_return_type(self):
        """Tests if max best options returns a dictionary"""
        maximum_best_score, maximum_best_options = self.g_unbound.search(maximum=True)
        self.assertIsInstance(maximum_best_options, dict)

class MethodReturnValues(Base):

    def test_search_greater_values(self):
        """Tests if max is greater than min in sample use-case"""
        minimum_best_score, minimum_best_options = self.g_unbound.search()
        maximum_best_score, maximum_best_options = self.g_unbound.search(maximum=True)
        self.assertGreater(maximum_best_score, minimum_best_score)

    def test_generate_grid_combinations(self):
        """Test that the number of combinations in grid equals
        the number parameter selection iterations specficied"""
        grid = self.g.generate_grid()
        self.assertEqual(len(grid), self.n_selection_iters)

    def test_generate_grid_param_mapping(self):
        """Test that generated values are correctly mapped to each parameter
        and are within the specified bounds
        """
        grid = self.g.generate_grid()
        for i in ['c1','c2','k','w']:
            values = [x[i] for x in grid]
            for j in values:
                self.assertGreaterEqual(j, self.options[i][0])
                self.assertLessEqual(j, self.options[i][1])

class Instantiation(Base):

    def test_optimizer_type_fail(self):
        """Test that :code:`optimizer` of type :code:`string` raises
        :code:`TypeError`"""
        bad_optimizer = 'LocalBestPSO'  # a string instead of a class object
        with self.assertRaises(TypeError):
            g = RandomSearch(bad_optimizer, self.n_particles, self.dimensions,
                             self.options, self.objective_func, self.iters,
                             self.n_selection_iters, bounds=None,
                             velocity_clamp=None)

    def test_n_selection_iters_type_fail(self):
        """Test that :code:`n_selection_iters` of type :code:`float` raises
        :code:`TypeError`"""
        bad_n_selection_iters = 100.0  # should be an int
        with self.assertRaises(TypeError):
            g = RandomSearch(self.optimizer, self.n_particles, self.dimensions,
                             self.options, self.objective_func, self.iters,
                             bad_n_selection_iters, self.bounds,
                             velocity_clamp=None)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" Unit testing for pyswarms.grid_search"""

# Import from __future__
from __future__ import with_statement
from __future__ import absolute_import
from __future__ import print_function

# Import modules
import unittest
import numpy as np

# Import from package
from pyswarms.utils.search.grid_search import GridSearch
from pyswarms.single import LocalBestPSO
from pyswarms.single import GlobalBestPSO
from pyswarms.utils.functions.single_obj import sphere_func

class Base(unittest.TestCase):

    def setUp(self):
        """Sets up test fixtures"""
        self.optimizer = LocalBestPSO
        self.n_particles = 40
        self.dimensions = 20
        self.options = {'c1': [1, 2, 3],
                        'c2': [1, 2, 3],
                        'k' : [5, 10, 15],
                        'w' : [0.9, 0.7, 0.4],
                        'p' : [1]}
        self.mini_options = {'c1': [1,2],
                   'c2': 6,
                   'k': 5,
                   'w': 0.9,
                   'p': 0}
        self.bounds = (np.array([-5,-5]), np.array([5,5]))
        self.iters = 10
        self.objective_func = sphere_func
        self.g = GridSearch(self.optimizer, self.n_particles, self.dimensions,
                       self.options, self.objective_func, self.iters,
                       bounds=None, velocity_clamp=None)
        self.g_mini = GridSearch(self.optimizer, self.n_particles, self.dimensions,
                       self.mini_options, self.objective_func, self.iters,
                       bounds=None, velocity_clamp=None)

class MethodReturnType(Base):

    def test_search_min_best_options_return_type(self):
        """Tests if best options returns a dictionary"""
        minimum_best_score, minimum_best_options = self.g.search()
        self.assertIsInstance(minimum_best_options, dict)

    def test_search_max_best_options_return_type(self):
        """Tests if max best options returns a dictionary"""
        maximum_best_score, maximum_best_options = self.g.search(maximum=True)
        self.assertIsInstance(maximum_best_options, dict)

class MethodReturnValues(Base):

    def test_search_greater_values(self):
        """Tests if max is greater than min in sample use-case"""
        minimum_best_score, minimum_best_options = self.g.search()
        maximum_best_score, maximum_best_options = self.g.search(maximum=True)
        self.assertGreater(maximum_best_score, minimum_best_score)

    def test_generate_grid(self):
        """Tests if generate_grid function returns expected value."""
        self.assertEqual(self.g_mini.generate_grid(),
                         [{'c1': 1, 'c2': 6, 'k': 5, 'w': 0.9, 'p': 0},
                          {'c1': 2, 'c2': 6, 'k': 5, 'w': 0.9, 'p': 0}])

class Instantiation(Base):

    def test_optimizer_type_fail(self):
        """Tests that :code:`optimizer` of type :code:`string` raises
        :code:`TypeError`"""
        bad_optimizer = 'LocalBestPSO'  # a string instead of a class object
        with self.assertRaises(TypeError):
            g = GridSearch(bad_optimizer, self.n_particles, self.dimensions,
                           self.options, self.objective_func, self.iters,
                           bounds=None, velocity_clamp=None)
